# -*- coding:utf-8 -*-
#
# Copyright © 2015 The Spyder Development Team
# Copyright © 2014 Gonzalo Peña-Castellanos (@goanpeca)
#
# Licensed under the terms of the MIT License

# -----------------------------------------------------------------------------
# The following statement is required to register this 3rd party plugin:
# -----------------------------------------------------------------------------
from .condapackages import CondaPackages as PLUGIN_CLASS  # analysis:ignore
